"""This module implements """
from typing import TypeVar

from fixturefilehandler import VacateFilePathInterface

TypeVarVacateFilePathInterface = TypeVar("TypeVarVacateFilePathInterface", bound=VacateFilePathInterface)
